@@ -1,8 +1,14 @@
-/* $Id: acconfig.h,v 1.9 2000/12/06 01:08:10 astyanax Exp $ */
+/* $Id: acconfig.h,v 1.14 2001/02/16 04:48:30 astyanax Exp $ */
 
 /* Define this if you have the wresize function in your ncurses-type library */
 #undef HAVE_WRESIZE
 
+/* Define this if you have the resizeterm function in your ncurses-type library */
+#undef HAVE_RESIZETERM
+
+/* Define this if your curses lib has the _use_keypad flag */
+#undef HAVE_USEKEYPAD
+
 /* Define this if you have NLS */
 #undef ENABLE_NLS
 
@@ -36,4 +42,8 @@
 /* Define this to disable the use(full|less) spelling functions */
 #undef DISABLE_SPELLER
 
+/* Define this to disable the ^G help menu */
+#undef DISABLE_HELP
 
+/* Define this to disable the built-in (crappy) file browser */
+#undef DISABLE_BROWSER
