@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.21 2000/10/26 01:43:04 astyanax Exp $ */
+/* $Id: cut.c,v 1.27 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -174,24 +174,23 @@ int do_cut_text(void)
     if (ISSET(MARK_ISSET)) {
 	if (current->lineno == mark_beginbuf->lineno) {
 	    tmp = copy_node(current);
-	    newsize = abs(strlen(&current->data[mark_beginx]) -
-			  strlen(&current->data[current_x]));
+	    newsize = abs(mark_beginx - current_x) + 1;
 
-	    tmpstr = nmalloc(newsize);
+	    tmpstr = nmalloc(newsize + 1);
 	    if (current_x < mark_beginx) {
 		strncpy(tmpstr, &current->data[current_x], newsize);
 		memmove(&current->data[current_x],
 			&current->data[mark_beginx],
-			strlen(&current->data[mark_beginx] - newsize));
+			strlen(&current->data[mark_beginx]) + 1);
 	    } else {
 		strncpy(tmpstr, &current->data[mark_beginx], newsize);
 		memmove(&current->data[mark_beginx],
 			&current->data[current_x],
-			strlen(&current->data[current_x] - newsize));
+			strlen(&current->data[current_x]) + 1);
 		current_x = mark_beginx;
 		update_cursor();
 	    }
-	    tmpstr[newsize] = 0;
+	    tmpstr[newsize - 1] = 0;
 	    tmp->data = tmpstr;
 	    add_to_cutbuffer(tmp);
 	    dump_buffer(cutbuffer);
@@ -274,6 +273,7 @@ int do_uncut_text(void)
     filestruct *tmp = current, *fileptr = current, *newbuf, *newend;
 #ifndef NANO_SMALL
     char *tmpstr, *tmpstr2;
+    filestruct *hold = current;
 #endif
     int i;
 
@@ -339,8 +339,13 @@ int do_uncut_text(void)
 
 	    if (tmp != NULL)
 		tmp->prev = newend;
-	    else
+	    else {
+		/* Fix the editbot pointer too */
+		if (editbot == filebot)
+		    editbot = newend;
 		filebot = newend;
+		new_magicline();
+	    }
 
 	    /* Now why don't we update the totsize also */
 	    for (tmp = current->next; tmp != newend; tmp = tmp->next)
@@ -368,7 +373,9 @@ int do_uncut_text(void)
 	    current_x = 0;
 	    placewewant = 0;
 	}
-	renumber(current->prev);
+	/* Renumber from BEFORE where we pasted ;) */
+	renumber(hold);
+
 	dump_buffer(fileage);
 	dump_buffer(cutbuffer);
 	set_modified();
