@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.55 2000/12/06 05:59:31 astyanax Exp $ */
+/* $Id: files.c,v 1.101 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <pwd.h>
 
 #include "config.h"
 #include "proto.h"
@@ -194,6 +195,7 @@ int read_file(int fd, char *filename)
     } else if (fileptr->next == NULL) {
 	filebot = fileptr;
 	new_magicline();
+	totsize--;
 
 	/* Update the edit buffer */
 	load_file();
@@ -226,10 +228,18 @@ int open_file(char *filename, int insert, int quiet)
     } else if ((fd = open(filename, O_RDONLY)) == -1) {
 	if (!quiet)
 	    statusbar("%s: %s", strerror(errno), filename);
+	if (!insert)
+	    new_file();
 	return -1;
     } else {			/* File is A-OK */
-	if (S_ISDIR(fileinfo.st_mode)) {
-	    statusbar(_("File \"%s\" is a directory"), filename);
+	if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) || 
+		S_ISBLK(fileinfo.st_mode)) {
+	    if (S_ISDIR(fileinfo.st_mode))
+		statusbar(_("File \"%s\" is a directory"), filename);
+	    else
+		/* Don't open character or block files.  Sorry, /dev/sndstat! */
+		statusbar(_("File \"%s\" is a device file"), filename);
+
 	    if (!insert)
 		new_file();
 	    return -1;
@@ -262,6 +272,21 @@ int do_insertfile(void)
 	realname = mallocstrcpy(realname, answer);
 #endif
 
+#ifndef DISABLE_BROWSER
+	if (i == NANO_TOFILES_KEY) {
+	    
+	    char *tmp = do_browse_from(realname);
+
+#ifdef DISABLE_TABCOMP
+	    realname = NULL;
+#endif
+	    if 	(tmp != NULL)
+		realname = mallocstrcpy(realname, tmp);
+	    else
+		return do_insertfile();
+	}
+#endif
+
 	i = open_file(realname, 1, 0);
 	free(realname);
 
@@ -299,22 +324,25 @@ int do_insertfile(void)
 int write_file(char *name, int tmp)
 {
     long size, lineswritten = 0;
-    char buf[PATH_MAX + 1];
+    static char *buf = NULL;
     filestruct *fileptr;
     int fd, mask = 0, realexists, anyexists;
-    struct stat st, lst, st2;
+    struct stat st, lst;
     static char *realname = NULL;
 
     if (!strcmp(name, "")) {
 	statusbar(_("Cancelled"));
 	return -1;
     }
-    titlebar();
+    titlebar(NULL);
     fileptr = fileage;
 
     if (realname != NULL)
 	free(realname);
 
+    if (buf != NULL)
+	free(buf);
+
 #ifndef DISABLE_TABCOMP
     realname = real_dir_from_tilde(name);
 #else
@@ -331,55 +359,39 @@ int write_file(char *name, int tmp)
     if (tmp && anyexists != -1)
 	return -1;
     /* NOTE: If you change this statement, you MUST CHANGE the if 
-       statement below (that starts "if ((!ISSET(FOLLOW_SYMLINKS)...")
+       statement below (that says:
+		if (realexists == -1 || tmp || (!ISSET(FOLLOW_SYMLINKS) &&
+		S_ISLNK(lst.st_mode))) {
        to reflect whether or not to link/unlink/rename the file */
     else if (ISSET(FOLLOW_SYMLINKS) || !S_ISLNK(lst.st_mode) || tmp) {
-
-	/* Use O_EXCL if tmp == 1, I suppose */
+	/* Use O_EXCL if tmp == 1.  This is now copied from joe, because
+	   wiggy says so *shrug*. */
 	if (tmp)
-	    fd = open(realname, O_WRONLY | O_CREAT | O_EXCL | O_TRUNC,
-		      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH |
-		      S_IWOTH);
+	    fd = open(realname, O_WRONLY | O_CREAT | O_EXCL, (S_IRUSR|S_IWUSR));
 	else
-	    fd = open(realname, O_WRONLY | O_CREAT | O_TRUNC,
-		      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH |
-		      S_IWOTH);
+	    fd = open(realname, O_WRONLY | O_CREAT | O_TRUNC, (S_IRUSR|S_IWUSR));
 
 	/* First, just give up if we couldn't even open the file */
 	if (fd == -1) {
 	    if (!tmp && ISSET(TEMP_OPT)) {
 		UNSET(TEMP_OPT);
-		return do_writeout(1);
+		return do_writeout(filename, 1);
 	    }
 	    statusbar(_("Could not open file for writing: %s"),
 		      strerror(errno));
-	    free(realname);
-	    return -1;
-	}
-
-	/* Now we fstat() the file, to make sure it's the same file still!
-	   Thanks to Oliver Friedrichs(?) for this code from securityfocus */
-
-	if (fstat(fd, &st2) != 0) {
-	    close(fd);
 	    return -1;
 	}
 
     }
     /* Don't follow symlink.  Create new file. */
     else {
-	if (strlen(realname) > (PATH_MAX - 7)) {
-	    statusbar(_("Could not open file: Path length exceeded."));
-	    return -1;
-	}
-
-	memset(buf, 0x00, PATH_MAX + 1);
-	strcat(buf, realname);
+	buf = nmalloc(strlen(realname) + 8);
+	strncpy(buf, realname, strlen(realname)+1);
 	strcat(buf, ".XXXXXX");
 	if ((fd = mkstemp(buf)) == -1) {
 	    if (ISSET(TEMP_OPT)) {
 		UNSET(TEMP_OPT);
-		return do_writeout(1);
+		return do_writeout(filename, 1);
 	    }
 	    statusbar(_("Could not open file for writing: %s"),
 		      strerror(errno));
@@ -475,22 +487,23 @@ int write_file(char *name, int tmp)
 		  mask, realname, strerror(errno));
 
     if (!tmp) {
-	strncpy(filename, realname, 132);
+	filename = mallocstrcpy(filename, realname);
 	statusbar(_("Wrote %d lines"), lineswritten);
 	UNSET(MODIFIED);
-	titlebar();
+	titlebar(NULL);
     }
     return 1;
 }
 
-int do_writeout(int exiting)
+int do_writeout(char *path, int exiting)
 {
     int i = 0;
+
 #ifdef NANO_EXTRA
     static int did_cred = 0;
 #endif
 
-    answer = mallocstrcpy(answer, filename);
+    answer = mallocstrcpy(answer, path);
 
     if ((exiting) && (ISSET(TEMP_OPT))) {
 	if (filename[0]) {
@@ -512,6 +525,18 @@ int do_writeout(int exiting)
 
 	if (i != -1) {
 
+#ifndef DISABLE_BROWSER
+	if (i == NANO_TOFILES_KEY) {
+
+	    char *tmp = do_browse_from(answer);
+
+	    if (tmp != NULL)
+		answer = mallocstrcpy(answer, tmp);
+	    else
+		return do_writeout(answer, exiting);
+	}
+#endif
+
 #ifdef DEBUG
 	    fprintf(stderr, _("filename is %s"), answer);
 #endif
@@ -547,85 +572,62 @@ int do_writeout(int exiting)
 
 int do_writeout_void(void)
 {
-    return do_writeout(0);
+    return do_writeout(filename, 0);
 }
 
 #ifndef DISABLE_TABCOMP
-static char **homedirs;
 
 /* Return a malloc()ed string containing the actual directory, used
  * to convert ~user and ~/ notation...
  */
 char *real_dir_from_tilde(char *buf)
 {
-    char *dirtmp = NULL, *line = NULL, byte[1], *lineptr;
-    int fd, i, status, searchctr = 1;
+    char *dirtmp = NULL, *find_user = NULL;
+    int i = 1;
+    struct passwd *userdata;
+
+    /* set a default value for dirtmp, in the case user home dir not found */
+    dirtmp = mallocstrcpy(dirtmp, buf);
 
     if (buf[0] == '~') {
-	if (buf[1] == '~')
-	    goto abort;		/* Handle ~~ without segfaulting =) */
-	else if (buf[1] == '/') {
+	if (buf[1] == 0 || buf[1] == '/') {
 	    if (getenv("HOME") != NULL) {
+
+		free(dirtmp);
 		dirtmp = nmalloc(strlen(buf) + 2 + strlen(getenv("HOME")));
 
-		sprintf(dirtmp, "%s/%s", getenv("HOME"), &buf[2]);
-	    }
-	} else if (buf[1] != 0) {
+		sprintf(dirtmp, "%s%s", getenv("HOME"), &buf[1]);
 
-	    if ((fd = open("/etc/passwd", O_RDONLY)) == -1)
-		goto abort;
+	    }
+	}
+	else {
 
 	    /* Figure how how much of of the str we need to compare */
-	    for (searchctr = 1; buf[searchctr] != '/' &&
-		 buf[searchctr] != 0; searchctr++);
+	    for (i = 1; buf[i] != '/' && buf[i] != 0; i++)
+		;
 
-	    do {
-		i = 0;
-		line = nmalloc(1);
-		while ((status = read(fd, byte, 1)) != 0
-		       && byte[0] != '\n') {
+	    find_user = mallocstrcpy(find_user, &buf[1]);
+	    find_user[i - 1] = 0;
 
-		    line[i] = byte[0];
-		    i++;
-		    line = nrealloc(line, i + 1);
-		}
-		line[i] = 0;
+	    for (userdata = getpwent(); userdata != NULL && 
+		  strcmp(userdata->pw_name, find_user); 
+		  userdata = getpwent());
 
-		if (i == 0)
-		    goto abort;
+	    free(find_user);
 
-		line[i] = 0;
-		lineptr = strtok(line, ":");
+	    if (userdata != NULL) {  /* User found */
 
-		if (!strncmp(lineptr, &buf[1], searchctr - 1)) {
+	        free(dirtmp);
+		dirtmp = nmalloc(strlen(buf) + 2 + strlen(userdata->pw_dir));
+		sprintf(dirtmp, "%s%s", userdata->pw_dir, &buf[i]);
 
-		    /* Okay, skip to the password portion now */
-		    for (i = 0; i <= 4 && lineptr != NULL; i++)
-			lineptr = strtok(NULL, ":");
+	    }
 
-		    if (lineptr == NULL)
-			goto abort;
-
-		    /* Else copy the new string into the new buf */
-		    dirtmp = nmalloc(strlen(buf) + 2 + strlen(lineptr));
-
-		    sprintf(dirtmp, "%s%s", lineptr, &buf[searchctr]);
-		    free(line);
-		    break;
-		}
-
-		free(line);
-
-	    } while (status != 0);
+	    endpwent();
 	}
-    } else
-	dirtmp = mallocstrcpy(dirtmp, buf);
+    }
 
     return dirtmp;
-
-  abort:
-    dirtmp = mallocstrcpy(dirtmp, buf);
-    return dirtmp;
 }
 
 /* Tack a slash onto the string we're completing if it's a directory */
@@ -674,79 +676,35 @@ int append_slash_if_dir(char *buf, int *lastWasTab, int *place)
 
 char **username_tab_completion(char *buf, int *num_matches)
 {
-    char **matches = (char **) NULL, *line = NULL, *lineptr;
-    char *matchline = NULL, *matchdir = NULL;
+    char **matches = (char **) NULL;
+    char *matchline = NULL;
+    struct passwd *userdata;
 
-    int fd, i = 0, status = 1;
-    char byte[1];
+    *num_matches = 0;
+    matches = nmalloc(BUFSIZ * sizeof(char *));
 
-    if ((fd = open("/etc/passwd", O_RDONLY)) == -1) {
-	return NULL;
-    }
-
-    if (homedirs != NULL) {
-	for (i = 0; i < *num_matches; i++)
-	    free(homedirs[i]);
-	free(homedirs);
-	homedirs = (char **) NULL;
-	*num_matches = 0;
-    }
-    matches = nmalloc(BUFSIZ);
-    homedirs = nmalloc(BUFSIZ);
     strcat(buf, "*");
-    do {
-	i = 0;
-	line = nmalloc(1);
-	while ((status = read(fd, byte, 1)) != 0 && byte[0] != '\n') {
 
-	    line[i] = byte[0];
-	    i++;
-	    line = nrealloc(line, i + 1);
-	}
+    while ((userdata = getpwent()) != NULL) {
 
-	if (i == 0)
-	    break;
-
-	line[i] = 0;
-	lineptr = strtok(line, ":");
-
-	if (check_wildcard_match(line, &buf[1]) == TRUE) {
-
-	    if (*num_matches == BUFSIZ)
-		break;
+	if (check_wildcard_match(userdata->pw_name, &buf[1]) == TRUE) {
 
 	    /* Cool, found a match.  Add it to the list
 	     * This makes a lot more sense to me (Chris) this way...
 	     */
-	    matchline = nmalloc(strlen(line) + 2);
-	    sprintf(matchline, "~%s", line);
 
-	    for (i = 0; i <= 4 && lineptr != NULL; i++)
-		lineptr = strtok(NULL, ":");
-
-	    if (lineptr == NULL)
-		break;
-
-	    matchdir = mallocstrcpy(matchdir, lineptr);
-	    homedirs[*num_matches] = matchdir;
+	    matchline = nmalloc(strlen(userdata->pw_name) + 2);
+	    sprintf(matchline, "~%s", userdata->pw_name);
 	    matches[*num_matches] = matchline;
-
 	    ++*num_matches;
 
 	    /* If there's no more room, bail out */
 	    if (*num_matches == BUFSIZ)
 		break;
 	}
+    }
+    endpwent();
 
-	free(line);
-
-    } while (status != 0);
-
-    close(fd);
-    return matches;
-#ifdef DEBUG
-    fprintf(stderr, "\nin username_tab_completion\n");
-#endif
     return (matches);
 }
 
@@ -760,14 +718,14 @@ char **cwd_tab_completion(char *buf, int *num_matches)
     DIR *dir;
     struct dirent *next;
 
-    matches = nmalloc(BUFSIZ);
+    matches = nmalloc(BUFSIZ * sizeof(char *));
 
     /* Stick a wildcard onto the buf, for later use */
     strcat(buf, "*");
 
     /* Okie, if there's a / in the buffer, strip out the directory part */
     if (strcmp(buf, "") && strstr(buf, "/")) {
-	dirName = malloc(strlen(buf) + 1);
+	dirName = nmalloc(strlen(buf) + 1);
 	tmp = buf + strlen(buf);
 	while (*tmp != '/' && tmp != buf)
 	    tmp--;
@@ -810,11 +768,6 @@ char **cwd_tab_completion(char *buf, int *num_matches)
     }
     while ((next = readdir(dir)) != NULL) {
 
-	/* Some quick sanity checks */
-	if ((strcmp(next->d_name, "..") == 0)
-	    || (strcmp(next->d_name, ".") == 0)) {
-	    continue;
-	}
 #ifdef DEBUG
 	fprintf(stderr, "Comparing \'%s\'\n", next->d_name);
 #endif
@@ -858,7 +811,8 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
 
 	/* Make a local copy of the string -- up to the position of the
 	   cursor */
-	matchBuf = (char *) calloc(strlen(buf) + 2, sizeof(char));
+	matchBuf = (char *) nmalloc((strlen(buf) + 2) * sizeof(char));
+	memset(matchBuf, '\0', (strlen(buf) + 2));
 
 	strncpy(matchBuf, buf, place);
 	tmp = matchBuf;
@@ -1032,3 +986,387 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
     return buf;
 }
 #endif
+
+#ifndef DISABLE_BROWSER
+
+/* Return the stat of the file pointed to by path */
+struct stat filestat(const char *path) {
+    struct stat st;
+
+    lstat(path, &st);
+    return st;
+}
+
+/* Our sort routine for file listings - sort directories before
+ * files, and then alphabetically
+ */ 
+int diralphasort(const void *va, const void *vb) {
+    struct stat file1info, file2info;
+    char *a = *(char **)va, *b = *(char **)vb;
+    int answer = 0;
+
+    if (stat(a, &file1info) == -1)
+	answer = 1;
+    else if (stat(b, &file2info) == -1)
+	answer = 1;
+    else {
+	/* If is a is a dir and b isn't, return -1.
+	   Else if b is a dir and a isn't, return 0.
+	   Else return a < b */
+
+	if (S_ISDIR(file1info.st_mode) && !S_ISDIR(file2info.st_mode))
+	    return -1;
+	else if (!S_ISDIR(file1info.st_mode) && S_ISDIR(file2info.st_mode))
+	    return 1;
+	else
+	    answer = strcmp(a, b);
+    }
+
+    return(answer);
+}
+
+
+/* Initialize the browser code, including the list of files in *path */
+char **browser_init(char *path, int *longest, int *numents)
+{
+    DIR *dir;
+    struct dirent *next;
+    char **filelist = (char **) NULL;
+    int i = 0;
+
+    dir = opendir(path);
+    if (!dir) 
+	return NULL;
+
+    *numents = 0;
+    while ((next = readdir(dir)) != NULL) {
+	if (!strcmp(next->d_name, "."))
+	   continue;
+	(*numents)++;
+	if (strlen(next->d_name) > *longest)
+	    *longest = strlen(next->d_name);
+    }
+    rewinddir(dir);
+    *longest += 10;
+
+    filelist = nmalloc(*numents * sizeof (char *));
+
+    while ((next = readdir(dir)) != NULL) {
+	if (!strcmp(next->d_name, "."))
+	   continue;
+	filelist[i] = nmalloc(strlen(next->d_name) + strlen(path) + 2);
+
+	if (!strcmp(path, "/"))
+	    snprintf(filelist[i], strlen(next->d_name) + strlen(path) + 1, 
+			"%s%s", path, next->d_name);
+	else
+	    snprintf(filelist[i], strlen(next->d_name) + strlen(path) + 2, 
+			"%s/%s", path, next->d_name);
+	i++;
+    }
+
+    if (*longest > COLS - 1)
+	*longest = COLS - 1;
+
+    return filelist;
+}
+
+/* Free our malloced memory */
+void free_charptrarray(char **array, int len)
+{
+    int i;
+
+    for (i = 0; i < len - 1; i++)
+	free(array[i]);
+    free(array);
+}
+
+/* only print the last part of a path, isn't there a shell 
+   command for this? */
+char *tail(char *foo)
+{
+    char *tmp = NULL;
+
+    tmp = foo + strlen(foo);
+    while (*tmp != '/' && tmp != foo)
+	tmp--;
+
+    tmp++;
+
+    return tmp;
+}
+
+/* Strip one dir from the end of a string */
+void striponedir(char *foo)
+{
+    char *tmp = NULL;
+
+    /* Don't strip the root dir */
+    if (!strcmp(foo, "/"))
+	return;
+
+    tmp = foo + strlen(foo);
+    if (*tmp == '/')
+	tmp--;
+
+    while (*tmp != '/' && tmp != foo)
+	tmp--;
+
+    if (tmp != foo)
+	*tmp = 0;
+    else
+	*(tmp+1) = 0;
+
+    return;
+}
+
+/* Our browser function.  inpath is the path to start browsing from */
+char *do_browser(char *inpath)
+{
+    struct stat st;
+    char *foo, *retval = NULL;
+    static char *path = NULL;
+    int numents = 0, i = 0, j = 0, kbinput = 0, longest = 0, abort = 0;
+    int col = 0, selected = 0, editline = 0, width = 0, filecols = 0;
+    int lineno = 0, kb;
+    char **filelist = (char **) NULL;
+
+    /* If path isn't the same as inpath, we are being passed a new
+	dir as an arg.  We free it here so it will be copied from 
+	inpath below */
+    if (path != NULL && strcmp(path, inpath)) {
+	free(path);
+	path = NULL;
+    }
+
+    /* if path doesn't exist, make it so */
+    if (path == NULL)
+	path = mallocstrcpy(path, inpath);
+
+    filelist = browser_init(path, &longest, &numents);
+    foo = nmalloc(longest + 8);
+
+    /* Sort the list by directory first then alphabetically */
+    qsort(filelist, numents, sizeof(char *), diralphasort);
+
+    kb = keypad_on(edit, 1);
+    titlebar(path);
+    bottombars(browser_list, BROWSER_LIST_LEN);
+    curs_set(0);
+    wmove(edit, 0, 0);
+    i = 0;
+    width = 0;
+    filecols = 0;
+
+    /* Loop invariant: Microsoft sucks. */
+    do {
+	blank_edit();
+	blank_statusbar();
+ 	editline = 0;
+	col = 0;
+	    
+	/* Compute line number we're on now so we don't divide by zero later */
+	if (width == 0)
+	    lineno = selected;
+	else
+	    lineno = selected / width;
+
+	switch (kbinput) {
+	case KEY_UP:
+	case 'u':
+	    if (selected - width >= 0)
+		selected -= width;
+	    break;
+	case KEY_LEFT:
+	case 'l':
+	    if (selected > 0)
+		selected--;
+	    break;
+	case KEY_DOWN:
+	case 'd':
+	    if (selected + width <= numents - 1)
+		selected += width;
+	    break;
+	case KEY_RIGHT:
+	case 'r':
+	    if (selected < numents - 1)
+		selected++;
+	    break;
+	case NANO_PREVPAGE_KEY:
+	case NANO_PREVPAGE_FKEY:
+	case KEY_PPAGE:
+	case '-':
+
+	    if (lineno % editwinrows == 0) {
+		if (selected - (editwinrows * width) >= 0)
+		    selected -= editwinrows * width; 
+		else
+		    selected = 0;
+	    }
+	    else if (selected - (editwinrows + 
+		lineno % editwinrows) * width  >= 0)
+
+		selected -= (editwinrows + lineno % editwinrows) * width; 
+	    else
+		selected = 0;
+	    break;
+	case NANO_NEXTPAGE_KEY:
+	case NANO_NEXTPAGE_FKEY:
+	case KEY_NPAGE:	
+	case ' ':
+	    if (lineno % editwinrows == 0) {
+		if (selected + (editwinrows * width) <= numents - 1)
+		    selected += editwinrows * width; 
+		else
+		    selected = numents - 1;
+	    }
+	    else if (selected + (editwinrows - 
+			lineno %  editwinrows) * width <= numents - 1)
+ 		selected += (editwinrows - lineno % editwinrows) * width; 
+ 	    else
+		selected = numents - 1;
+	    break;
+	case KEY_ENTER:
+	case NANO_CONTROL_M:
+	case 's': /* More Pico compatibility */
+	case 'S':
+
+	    /* You can't cd up from / */
+	    if (!strcmp(filelist[selected], "/..") && !strcmp(path, "/"))
+		statusbar(_("Can't move up a directory"));
+	    else
+		path = mallocstrcpy(path, filelist[selected]);
+
+	    st = filestat(path);
+	    if (S_ISDIR(st.st_mode)) {
+		if (opendir(path) == NULL) {
+		    /* We can't open this dir for some reason.  Complain */
+		    statusbar(_("Can't open \"%s\": %s"), path, strerror(errno));
+		    striponedir(path);		    
+		    align(&path);
+		    break;
+		} 
+
+		if (!strcmp("..", tail(path))) {
+		    /* They want to go up a level, so strip off .. and the
+			current dir */
+		    striponedir(path);
+		    striponedir(path);
+		    align(&path);
+		}
+
+		/* Start over again with the new path value */
+		return do_browser(path);
+	    } else {
+		retval = path;
+		abort = 1;
+	    }
+	    break;
+	/* Stuff we want to abort the browser */
+	case 'q':
+	case 'Q':
+	case 'e':	/* Pico compatibility, yeech */
+	case 'E':
+	case NANO_EXIT_FKEY:
+		abort = 1;
+		break;
+	}
+	if (abort)
+	    break;
+
+	if (width)
+	    i = width * editwinrows * ((selected / width) / editwinrows);
+	else
+	    i = 0;
+
+	wmove(edit, 0, 0);
+	for (j = i; j < numents && editline <= editwinrows - 1; j++) {
+	    filecols++;
+
+	    strncpy(foo, tail(filelist[j]), strlen(tail(filelist[j])) + 1);
+	    while (strlen(foo) < longest)
+		strcat(foo, " ");
+	    col += strlen(foo);
+
+	    /* Put file info in the string also */
+	    st = filestat(filelist[j]);
+	    if (S_ISDIR(st.st_mode))
+		strcpy(foo + longest - 5, "(dir)");
+	    else {
+		if (S_ISLNK(st.st_mode))
+		    strcpy(foo + longest - 2, "--");
+		else if (st.st_size < 1024) /* less than 1 K */
+		    sprintf(foo + longest - 7, "%4d  B", (int) st.st_size);
+		else if (st.st_size > 1073741824) /* at least 1 gig */
+		    sprintf(foo + longest - 7, "%4d GB", (int) st.st_size / 1073741824);
+		else if (st.st_size > 1048576) /* at least 1 meg */
+		    sprintf(foo + longest - 7, "%4d MB", (int) st.st_size / 1048576);
+		else /* Its more than 1 k and less than a meg */
+		    sprintf(foo + longest - 7, "%4d KB", (int) st.st_size / 1024);
+	    }
+
+	    /* Hilight the currently selected file/dir */
+	    if (j == selected)
+		wattron(edit, A_REVERSE);
+	    waddnstr(edit, foo, strlen(foo));
+	    if (j == selected)
+		wattroff(edit, A_REVERSE);
+
+	    /* And add some space between the cols */
+	    waddstr(edit, "  ");
+	    col += 2;
+
+	    /* And if the next entry isn't going to fit on the
+		line, move to the next one */
+	    if (col > (COLS - longest)) {
+		editline++;
+		wmove(edit, editline, 0);
+		col = 0;
+		if (width == 0)
+		    width = filecols;
+	    }
+	}
+ 	wrefresh(edit);
+    } while ((kbinput = wgetch(edit)) != NANO_EXIT_KEY);
+    curs_set(1);
+    blank_edit();
+    titlebar(NULL); 
+    edit_refresh();
+    kb = keypad_on(edit, kb);
+
+    /* cleanup */
+    free_charptrarray(filelist, numents);
+    free(foo);
+    return retval;
+}
+
+/* Browser fron't end, checks to see if inpath has a dir in it and if so
+ starts do_browser from there, else from the current dir */
+char *do_browse_from(char *inpath)
+{
+    struct stat st;
+    char *tmp = NULL;
+
+    tmp = mallocstrcpy(tmp, inpath);
+
+    /* If there's no / in the string, we may was well start from . */
+    if (tmp == NULL || !strstr(tmp, "/"))
+	return do_browser(getcwd(NULL, 0));
+
+    /* If the string is a directory, pass do_browser that */
+    st = filestat(tmp);
+    if (S_ISDIR(st.st_mode))
+	return do_browser(tmp);
+
+    /* Okay, there's a dir in there, but not at the end of the string... 
+       try stripping it off */
+    striponedir(tmp);
+    align(&tmp);
+    return do_browser(tmp);
+
+}
+
+
+
+#endif
+
