@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.19 2000/11/29 23:57:20 astyanax Exp $ */
+/* $Id: global.c,v 1.30 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -35,11 +35,10 @@
  * Global variables
  */
 int flags = 0;			/* Our new flag containig many options */
-int center_x = 0, center_y = 0;	/* Center of screen */
 WINDOW *edit;			/* The file portion of the editor  */
 WINDOW *topwin;			/* Top line of screen */
 WINDOW *bottomwin;		/* Bottom buffer */
-char filename[PATH_MAX];	/* Name of the file */
+char *filename = NULL;		/* Name of the file */
 int editwinrows = 0;		/* How many rows long is the edit
 				   window? */
 filestruct *current;		/* Current buffer pointer */
@@ -79,7 +78,13 @@ shortcut goto_list[GOTO_LIST_LEN];
 shortcut writefile_list[WRITEFILE_LIST_LEN];
 shortcut help_list[HELP_LIST_LEN];
 shortcut spell_list[SPELL_LIST_LEN];
+#ifndef DISABLE_BROWSER
+shortcut browser_list[BROWSER_LIST_LEN];
+#endif
+
+#ifndef NANO_SMALL
 toggle toggles[TOGGLE_LEN];
+#endif
 
 /* Regular expressions */
 
@@ -103,6 +108,7 @@ void sc_init_one(shortcut * s, int key, char *desc, char *help, int alt,
     s->func = func;
 }
 
+#ifndef NANO_SMALL
 /* Initialize the toggles in the same manner */
 void toggle_init_one(toggle * t, int val, char *desc, int flag)
 {
@@ -110,6 +116,7 @@ void toggle_init_one(toggle * t, int val, char *desc, int flag)
     t->desc = desc;
     t->flag = flag;
 }
+#endif
 
 void toggle_init(void)
 {
@@ -170,9 +177,11 @@ void shortcut_init(int unjustify)
 	"", *nano_mark_msg = "", *nano_delete_msg =
 	"", *nano_backspace_msg = "", *nano_tab_msg =
 	"", *nano_enter_msg = "", *nano_case_msg =
-	"", *nano_cancel_msg = "", *nano_unjustify_msg;
+	"", *nano_cancel_msg = "", *nano_unjustify_msg = "";
 
 #ifndef NANO_SMALL
+    char *nano_tofiles_msg = "";
+
     nano_help_msg = _("Invoke the help menu");
     nano_writeout_msg = _("Write the current file to disk");
     nano_exit_msg = _("Exit from nano");
@@ -205,42 +214,39 @@ void shortcut_init(int unjustify)
     nano_enter_msg = _("Insert a carriage return at the cursor position");
     nano_case_msg =
 	_("Make the current search or replace case (in)sensitive");
+    nano_tofiles_msg = _("Go to file browser");
     nano_cancel_msg = _("Cancel the current function");
 #endif
 
-    if (ISSET(PICO_MODE))
 	sc_init_one(&main_list[0], NANO_HELP_KEY, _("Get Help"),
 		    nano_help_msg, 0, NANO_HELP_FKEY, 0, VIEW, do_help);
-    else
-	sc_init_one(&main_list[0], NANO_WRITEOUT_KEY, _("WriteOut"),
-		    nano_writeout_msg,
-		    0, NANO_WRITEOUT_FKEY, 0, NOVIEW, do_writeout_void);
 
     sc_init_one(&main_list[1], NANO_EXIT_KEY, _("Exit"),
 		nano_exit_msg, 0, NANO_EXIT_FKEY, 0, VIEW, do_exit);
 
-    if (ISSET(PICO_MODE))
-	sc_init_one(&main_list[2], NANO_WRITEOUT_KEY, _("WriteOut"),
+    sc_init_one(&main_list[2], NANO_WRITEOUT_KEY, _("WriteOut"),
 		    nano_writeout_msg,
 		    0, NANO_WRITEOUT_FKEY, 0, NOVIEW, do_writeout_void);
-    else
-	sc_init_one(&main_list[2], NANO_GOTO_KEY, _("Goto Line"),
-		    nano_goto_msg,
-		    NANO_ALT_G, NANO_GOTO_FKEY, 0, VIEW, do_gotoline_void);
 
     if (ISSET(PICO_MODE))
 	sc_init_one(&main_list[3], NANO_JUSTIFY_KEY, _("Justify"),
 		    nano_justify_msg, 0, NANO_JUSTIFY_FKEY, 0,
 		    NOVIEW, do_justify);
     else
-	sc_init_one(&main_list[3], NANO_REPLACE_KEY, _("Replace"),
-		    nano_replace_msg,
-		    NANO_ALT_R, NANO_REPLACE_FKEY, 0, NOVIEW, do_replace);
-
-    sc_init_one(&main_list[4], NANO_INSERTFILE_KEY, _("Read File"),
+	sc_init_one(&main_list[3], NANO_INSERTFILE_KEY, _("Read File"),
 		nano_insert_msg,
 		0, NANO_INSERTFILE_FKEY, 0, NOVIEW, do_insertfile);
 
+
+    if (ISSET(PICO_MODE))
+	sc_init_one(&main_list[4], NANO_INSERTFILE_KEY, _("Read File"),
+		nano_insert_msg,
+		0, NANO_INSERTFILE_FKEY, 0, NOVIEW, do_insertfile);
+    else
+	sc_init_one(&main_list[4], NANO_REPLACE_KEY, _("Replace"),
+		    nano_replace_msg,
+		    NANO_ALT_R, NANO_REPLACE_FKEY, 0, NOVIEW, do_replace);
+
     sc_init_one(&main_list[5], NANO_WHEREIS_KEY, _("Where Is"),
 		nano_whereis_msg,
 		0, NANO_WHEREIS_FKEY, 0, VIEW, do_search);
@@ -320,13 +326,10 @@ void shortcut_init(int unjustify)
 		nano_enter_msg,
 		0, KEY_ENTER, NANO_CONTROL_M, NOVIEW, do_enter_void);
 
-    if (ISSET(PICO_MODE))
-	sc_init_one(&main_list[25], NANO_GOTO_KEY, _("Goto Line"),
+    sc_init_one(&main_list[25], NANO_GOTO_KEY, _("Goto Line"),
 		    nano_goto_msg,
 		    NANO_ALT_G, NANO_GOTO_FKEY, 0, VIEW, do_gotoline_void);
-    else
-	sc_init_one(&main_list[25], NANO_HELP_KEY, _("Get Help"),
-		    nano_help_msg, 0, NANO_HELP_FKEY, 0, VIEW, do_help);
+
 
 
     sc_init_one(&whereis_list[0], NANO_FIRSTLINE_KEY, _("First Line"),
@@ -403,15 +406,31 @@ void shortcut_init(int unjustify)
 		nano_exit_msg, 0, NANO_EXIT_FKEY, 0, VIEW, do_exit);
 
 
-    sc_init_one(&writefile_list[0], NANO_CANCEL_KEY, _("Cancel"),
-		nano_cancel_msg, 0, 0, 0, VIEW, 0);
+#ifndef DISABLE_BROWSER
+    sc_init_one(&writefile_list[0], NANO_TOFILES_KEY, _("To Files"),
+		nano_tofiles_msg, 0, 0, 0, NOVIEW, 0);
+#endif
 
-
-    sc_init_one(&writefile_list[0], NANO_CANCEL_KEY, _("Cancel"),
+    sc_init_one(&writefile_list[WRITEFILE_LIST_LEN - 1], NANO_CANCEL_KEY, _("Cancel"),
 		nano_cancel_msg, 0, 0, 0, VIEW, 0);
 
     sc_init_one(&spell_list[0], NANO_CANCEL_KEY, _("Cancel"),
 		nano_cancel_msg, 0, 0, 0, VIEW, 0);
 
+#ifndef DISABLE_BROWSER
+    sc_init_one(&browser_list[0], NANO_PREVPAGE_KEY, _("Prev Page"),
+		nano_prevpage_msg,
+		0, NANO_PREVPAGE_FKEY, KEY_PPAGE, VIEW, 0);
+
+    sc_init_one(&browser_list[1], NANO_NEXTPAGE_KEY, _("Next Page"),
+		nano_nextpage_msg,
+		0, NANO_NEXTPAGE_FKEY, KEY_NPAGE, VIEW, 0);
+
+    sc_init_one(&browser_list[2], NANO_EXIT_KEY, _("Exit"),
+		nano_exit_msg, 0, NANO_EXIT_FKEY, 0, VIEW, 0);
+
+#endif
+
+
     toggle_init();
 }
