@@ -1,4 +1,4 @@
-/* $Id: move.c,v 1.12 2000/11/23 01:19:58 astyanax Exp $ */
+/* $Id: move.c,v 1.18 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
@@ -55,7 +55,12 @@ int page_down(void)
     if (current == filebot)
 	return 0;
 
-    if (editbot != filebot || edittop == fileage) {
+    /* AHEM, if we only have a screen or less of text, DONT do an
+	edit_update, just move the cursor to editbot! */
+    if (edittop == fileage && editbot == filebot && totlines < editwinrows) {
+	current = editbot;
+	reset_cursor();
+    } else if (editbot != filebot || edittop == fileage) {
 	current_y = 0;
 	current = editbot;
 
