@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.95 2000/12/06 01:08:10 astyanax Exp $ */
+/* $Id: nano.c,v 1.143 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <signal.h>
+#include <setjmp.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
@@ -62,35 +63,37 @@
 
 /* Former globals, now static */
 int fill = 0;			/* Fill - where to wrap lines, basically */
+
+#ifndef DISABLE_SPELLER
 static char *alt_speller;	/* Alternative spell command */
+#endif
+
 struct termios oldterm;		/* The user's original term settings */
-static char *help_text_init = "";
-				/* Initial message, not including shortcuts */
 static struct sigaction act;	/* For all out fun signal handlers */
 
+#ifndef DISABLE_HELP
+static char *help_text_init = ""; /* Initial message, not including shortcuts */
+#endif
+
 char *last_search = NULL;	/* Last string we searched for */
 char *last_replace = NULL;	/* Last replacement string */
 int search_last_line;		/* Is this the last search line? */
 
-void keypad_on(int yesno)
-{
-    keypad(edit, yesno);
-    keypad(bottomwin, yesno);
-}
+static sigjmp_buf jmpbuf;	/* Used to return to mainloop after SIGWINCH */
 
 /* What we do when we're all set to exit */
 RETSIGTYPE finish(int sigage)
 {
+
+    keypad(edit, TRUE);
+    keypad(bottomwin, TRUE);
+
     if (!ISSET(NO_HELP)) {
 	mvwaddstr(bottomwin, 1, 0, hblank);
 	mvwaddstr(bottomwin, 2, 0, hblank);
     } else
 	mvwaddstr(bottomwin, 0, 0, hblank);
     
-    /* Apparently you REALLY can't get away with not calling keypad()
-       or your window looks awful when it exits.  so we just call it right
-       before we exit, muhaha :-) */
-    keypad_on(TRUE);
     wrefresh(bottomwin);
     endwin();
 
@@ -141,22 +144,38 @@ void die(char *msg, ...)
     exit(1);			/* We have a problem: exit w/ errorlevel(1) */
 }
 
+/* Die with an error message that the screen was too small if, well, the
+   screen is too small */
+void die_too_small(void)
+{
+    char *too_small_msg = _("Window size is too small for Nano...");
+
+    die(too_small_msg);
+
+}
+
 void print_view_warning(void)
 {
     statusbar(_("Key illegal in VIEW mode"));
 }
 
+void clear_filename(void)
+{
+    if (filename != NULL)
+	free(filename);
+    filename = nmalloc(1);
+    filename[0] = 0;
+}
 
 /* Initialize global variables - no better way for now */
 void global_init(void)
 {
-    int i;
-
-    center_x = COLS / 2;
-    center_y = LINES / 2;
     current_x = 0;
     current_y = 0;
-    editwinrows = LINES - 5 + no_help();
+
+    if ((editwinrows = LINES - 5 + no_help()) < MIN_EDITOR_ROWS)
+	die_too_small();
+
     fileage = NULL;
     cutbuffer = NULL;
     current = NULL;
@@ -164,21 +183,22 @@ void global_init(void)
     editbot = NULL;
     totlines = 0;
     placewewant = 0;
-    if (!fill)
-	fill = COLS - 8;
-    hblank = nmalloc(COLS + 1);
 
-    /* Thanks BG for this bit... */
-    for (i = 0; i <= COLS - 1; i++)
-	hblank[i] = ' ';
-    hblank[i] = 0;
+    if (!fill)
+	fill = COLS - CHARS_FROM_EOL;
+
+    if (fill < MIN_FILL_LENGTH)
+	die_too_small();
+
+    hblank = nmalloc(COLS + 1);
+    memset(hblank, ' ', COLS);
+    hblank[COLS] = 0;
 }
 
+#ifndef DISABLE_HELP
 void init_help_msg(void)
 {
 
-#ifndef NANO_SMALL
-
     help_text_init =
 	_(" nano help text\n\n "
 	  "The nano editor is designed to emulate the functionality and "
@@ -197,9 +217,9 @@ void init_help_msg(void)
 	  "Esc, Alt or Meta key depending on your keyboard setup.  The "
 	  "following keystrokes are available in the main editor window. "
 	  "Optional keys are shown in parentheses:\n\n");
-#endif
 
 }
+#endif
 
 /* Make a copy of a node to a pointer (space will be malloc()ed) */
 filestruct *copy_node(filestruct * src)
@@ -230,8 +250,11 @@ void unlink_node(filestruct * fileptr)
 
 void delete_node(filestruct * fileptr)
 {
+    if (fileptr == NULL)
+	return;
+
     if (fileptr->data != NULL)
-	free(fileptr->data);
+        free(fileptr->data);
     free(fileptr);
 }
 
@@ -259,18 +282,6 @@ filestruct *copy_filestruct(filestruct * src)
     return head;
 }
 
-/* Free() a single node */
-int free_node(filestruct * src)
-{
-    if (src == NULL)
-	return 0;
-
-    if (src->next != NULL)
-	free(src->data);
-    free(src);
-    return 1;
-}
-
 int free_filestruct(filestruct * src)
 {
     filestruct *fileptr = src;
@@ -280,15 +291,15 @@ int free_filestruct(filestruct * src)
 
     while (fileptr->next != NULL) {
 	fileptr = fileptr->next;
-	free_node(fileptr->prev);
+	delete_node(fileptr->prev);
 
 #ifdef DEBUG
-	fprintf(stderr, _("free_node(): free'd a node, YAY!\n"));
+	fprintf(stderr, _("delete_node(): free'd a node, YAY!\n"));
 #endif
     }
-    free_node(fileptr);
+    delete_node(fileptr);
 #ifdef DEBUG
-    fprintf(stderr, _("free_node(): free'd last node.\n"));
+    fprintf(stderr, _("delete_node(): free'd last node.\n"));
 #endif
 
     return 1;
@@ -315,7 +326,10 @@ int renumber(filestruct * fileptr)
 	return 0;
     }
     for (temp = fileptr; temp != NULL; temp = temp->next) {
-	temp->lineno = temp->prev->lineno + 1;
+	if (temp->prev != NULL)
+	    temp->lineno = temp->prev->lineno + 1;
+	else
+	    temp->lineno = 1;
     }
 
     return 0;
@@ -343,7 +357,7 @@ void usage(void)
     printf(_("Usage: nano [GNU long option] [option] +LINE <file>\n\n"));
     printf(_("Option		Long option		Meaning\n"));
     printf(_
-	   (" -T 		--tabsize=[num]		Set width of a tab to num\n"));
+	   (" -T [num]	--tabsize=[num]		Set width of a tab to num\n"));
 #ifdef HAVE_REGEX_H
     printf(_
 	   (" -R		--regexp		Use regular expressions for search\n"));
@@ -355,12 +369,12 @@ void usage(void)
 	   (" -c 		--const			Constantly show cursor position\n"));
     printf(_
 	   (" -h 		--help			Show this message\n"));
+    printf(_
+	   (" -i 		--autoindent		Automatically indent new lines\n"));
 #ifndef NANO_SMALL
     printf(_
 	   (" -k 		--cut			Let ^K cut from cursor to end of line\n"));
 #endif
-    printf(_
-	   (" -i 		--autoindent		Automatically indent new lines\n"));
     printf(_
 	   (" -l 		--nofollow		Don't follow symbolic links, overwrite\n"));
 #ifndef NANO_SMALL
@@ -368,13 +382,15 @@ void usage(void)
     printf(_(" -m 		--mouse			Enable mouse\n"));
 #endif
 #endif
+    printf(_
+	   (" -p	 	--pico			Emulate Pico as closely as possible\n"));
     printf
 	(_
 	 (" -r [#cols] 	--fill=[#cols]		Set fill cols to (wrap lines at) #cols\n"));
-    printf(_
-	   (" -p	 	--pico			Emulate Pico as closely as possible\n"));
+#ifndef DISABLE_SPELLER
     printf(_
 	   (" -s [prog] 	--speller=[prog]	Enable alternate speller\n"));
+#endif
     printf(_
 	   (" -t 		--tempfile		Auto save on exit, don't prompt\n"));
     printf(_
@@ -395,10 +411,10 @@ void usage(void)
     printf(_(" -V 		Print version information and exit\n"));
     printf(_(" -c 		Constantly show cursor position\n"));
     printf(_(" -h 		Show this message\n"));
+    printf(_(" -i 		Automatically indent new lines\n"));
 #ifndef NANO_SMALL
     printf(_(" -k 		Let ^K cut from cursor to end of line\n"));
 #endif
-    printf(_(" -i 		Automatically indent new lines\n"));
     printf(_
 	   (" -l 		Don't follow symbolic links, overwrite\n"));
 #ifndef NANO_SMALL
@@ -406,10 +422,11 @@ void usage(void)
     printf(_(" -m 		Enable mouse\n"));
 #endif
 #endif
-    printf(_
-	   (" -r [#cols] 	Set fill cols to (wrap lines at) #cols\n"));
-    printf(_(" -s [prog]  	Enable alternate speller\n"));
     printf(_(" -p 		Emulate Pico as closely as possible\n"));
+    printf(_(" -r [#cols] 	Set fill cols to (wrap lines at) #cols\n"));
+#ifndef DISABLE_SPELLER
+    printf(_(" -s [prog]  	Enable alternate speller\n"));
+#endif
     printf(_(" -t 		Auto save on exit, don't prompt\n"));
     printf(_(" -v 		View (read only) mode\n"));
     printf(_(" -w 		Don't wrap long lines\n"));
@@ -422,27 +439,36 @@ void usage(void)
 
 void version(void)
 {
-    printf(_(" nano version %s by Chris Allegretta (compiled %s, %s)\n"),
+    printf(_(" GNU nano version %s (compiled %s, %s)\n"),
 	   VERSION, __TIME__, __DATE__);
     printf(_
 	   (" Email: nano@nano-editor.org	Web: http://www.nano-editor.org"));
     printf(_("\n Compiled options:"));
 
-#ifdef NANO_SMALL
-    printf(" --enable-tiny");
-#endif
 #ifdef NANO_EXTRA
     printf(" --enable-extra");
 #endif
-#ifdef DISABLE_TABCOMP
+
+#ifdef NANO_SMALL
+    printf(" --enable-tiny");
+#else
+ #ifdef DISABLE_BROWSER
+    printf(" --disable-browser");
+ #endif
+ #ifdef DISABLE_TABCOMP
     printf(" --disable-tabcomp");
-#endif
-#ifdef DISABLE_JUSTIFY
+ #endif
+ #ifdef DISABLE_JUSTIFY
     printf(" --disable-justify");
-#endif
-#ifdef DISABLE_SPELLER
+ #endif
+ #ifdef DISABLE_SPELLER
     printf(" --disable-speller");
+ #endif
+ #ifdef DISABLE_HELP
+    printf(" --disable-help");
+ #endif
 #endif
+
 #ifdef USE_SLANG
     printf(" --with-slang");
 #endif
@@ -467,19 +493,19 @@ filestruct *make_new_node(filestruct * prevnode)
 }
 
 /* Splice a node into an existing filestruct */
-void splice_node(filestruct * begin, filestruct * new, filestruct * end)
+void splice_node(filestruct * begin, filestruct * newnode, filestruct * end)
 {
-    new->next = end;
-    new->prev = begin;
-    begin->next = new;
+    newnode->next = end;
+    newnode->prev = begin;
+    begin->next = newnode;
     if (end != NULL)
-	end->prev = new;
+	end->prev = newnode;
 }
 
-int do_mark()
+int do_mark(void)
 {
 #ifdef NANO_SMALL
-    nano_small_msg();
+    nano_disabled_msg();
 #else
     if (!ISSET(MARK_ISSET)) {
 	statusbar(_("Mark Set"));
@@ -507,14 +533,7 @@ int no_help(void)
 	return 0;
 }
 
-#ifdef NANO_SMALL
-void nano_small_msg(void)
-{
-    statusbar("Sorry, this function not available with nano-tiny option");
-}
-#endif
-
-#if defined(DISABLE_JUSTIFY) || defined(DISABLE_SPELLER)
+#if defined(DISABLE_JUSTIFY) || defined(DISABLE_SPELLER) || defined(DISABLE_HELP)
 void nano_disabled_msg(void)
 {
     statusbar("Sorry, support for this function has been disabled");
@@ -551,11 +570,11 @@ void do_char(char ch)
 /* Someone hits return *gasp!* */
 int do_enter(filestruct * inptr)
 {
-    filestruct *new;
+    filestruct *newnode;
     char *tmp, *spc;
     int extra = 0;
 
-    new = make_new_node(inptr);
+    newnode = make_new_node(inptr);
     tmp = &current->data[current_x];
     current_x = 0;
 
@@ -567,26 +586,27 @@ int do_enter(filestruct * inptr)
 		extra++;
 		spc++;
 		current_x++;
+		totsize++;
 	    }
-	    new->data = nmalloc(strlen(tmp) + extra + 1);
-	    strncpy(new->data, current->data, extra);
-	    strcpy(&new->data[extra], tmp);
+	    newnode->data = nmalloc(strlen(tmp) + extra + 1);
+	    strncpy(newnode->data, current->data, extra);
+	    strcpy(&newnode->data[extra], tmp);
 	}
     } else {
-	new->data = nmalloc(strlen(tmp) + 1);
-	strcpy(new->data, tmp);
+	newnode->data = nmalloc(strlen(tmp) + 1);
+	strcpy(newnode->data, tmp);
     }
     *tmp = 0;
 
     if (inptr->next == NULL) {
-	filebot = new;
-	editbot = new;
+	filebot = newnode;
+	editbot = newnode;
     }
-    splice_node(inptr, new, inptr->next);
+    splice_node(inptr, newnode, inptr->next);
 
     totsize++;
     renumber(current);
-    current = new;
+    current = newnode;
     align(&current->data);
 
     /* The logic here is as follows:
@@ -785,6 +805,14 @@ void do_wrap(filestruct * inptr, char input_char)
 	if (current_x >= current_word_start) {
 	    right = current_x - current_word_start;
 	    current_x = 0;
+	    if (ISSET(AUTOINDENT)) {
+		int i = 0;
+		while ((inptr->next->data[i] == ' ' 
+	    		|| inptr->next->data[i] == '\t')) {
+		    i++;
+		    right++;
+		}			
+	    }
 	    down = 1;
 	}
 
@@ -798,7 +826,6 @@ void do_wrap(filestruct * inptr, char input_char)
 
     /* Category 2: two or more words on the line. */
     else {
-
 	/* Case 2a: cursor before word at wrap point. */
 	if (current_x < current_word_start) {
 	    temp->data =
@@ -831,6 +858,15 @@ void do_wrap(filestruct * inptr, char input_char)
 	    down = 1;
 
 	    right = current_x - current_word_start;
+	    if (ISSET(AUTOINDENT)) {
+		int i = 0;
+		while ((inptr->next->data[i] == ' ' 
+	    		|| inptr->next->data[i] == '\t')) {
+		    i++;
+		    right++;
+		}			
+	    }
+
 	    i = current_word_start - 1;
 	    if (isspace((int) input_char)
 		&& (current_x == current_word_start)) {
@@ -872,14 +908,34 @@ void do_wrap(filestruct * inptr, char input_char)
 
     /* We pre-pend wrapped part to next line. */
     if (ISSET(SAMELINEWRAP) && inptr->next) {
-	/* Plus one for the space which concatenates the two lines together plus 1 for \0. */
-	char *p =
-	    nmalloc(strlen(temp->data) + strlen(inptr->next->data) + 2);
 	int old_x = current_x, old_y = current_y;
 
-	strcpy(p, temp->data);
-	strcat(p, " ");
-	strcat(p, inptr->next->data);
+	/* Plus one for the space which concatenates the two lines together plus 1 for \0. */
+	char *p = nmalloc((strlen(temp->data) + strlen(inptr->next->data) + 2) 
+			* sizeof(char));
+
+	if (ISSET(AUTOINDENT)) {
+	    int non = 0;
+
+	     /* Grab the beginning of the next line until it's not a 
+		space or tab, then null terminate it so we can strcat it
+		to hell */
+	    while ((inptr->next->data[non] == ' ' 
+	    	|| inptr->next->data[non] == '\t'))
+	    	    p[non] = inptr->next->data[non++];
+
+	    p[non] = 0;
+	    strcat(p, temp->data);
+	    strcat(p, " ");
+
+	     /* Now tack on the rest of the next line after the spaces and
+		tabs */
+	    strcat(p, &inptr->next->data[non]);
+	} else {
+	    strcpy(p, temp->data);
+	    strcat(p, " ");
+	    strcat(p, inptr->next->data);
+	}
 
 	free(inptr->next->data);
 	inptr->next->data = p;
@@ -892,6 +948,7 @@ void do_wrap(filestruct * inptr, char input_char)
     }
     /* Else we start a new line. */
     else {
+
 	temp->prev = inptr;
 	temp->next = inptr->next;
 
@@ -903,6 +960,24 @@ void do_wrap(filestruct * inptr, char input_char)
 	    filebot = temp;
 
 	SET(SAMELINEWRAP);
+
+	if (ISSET(AUTOINDENT)) {
+	    char *spc = inptr->data;
+	    char *t = NULL;
+	    int extra = 0;
+	    if (spc) {
+	        while ((*spc == ' ') || (*spc == '\t')) {
+		    extra++;
+		    spc++;
+		    totsize++;
+		}
+		t = nmalloc(strlen(temp->data) + extra + 1);
+		strncpy(t, inptr->data, extra);
+		strcpy(t + extra, temp->data);
+		free(temp->data);
+		temp->data = t;
+	    }
+	}
     }
 
 
@@ -910,7 +985,7 @@ void do_wrap(filestruct * inptr, char input_char)
     /* Everything about it makes me want this line here but it causes
      * totsize to be high by one for some reason.  Sigh. (Rob) */
     /* totsize++; */
-
+	
     renumber(inptr);
     edit_update(edittop, TOP);
 
@@ -1030,7 +1105,6 @@ int do_backspace(void)
 	    if (strcmp(current->data, "")) {
 		new_magicline();
 		fix_editbot();
-		totsize++;
 	    }
 	}
 
@@ -1062,7 +1136,9 @@ int do_delete(void)
 
 	align(&current->data);
 
-    } else if (current->next != NULL) {
+    /* Now that we have a magic lnie again, we can check for both being
+       on the line before filebot as well as at filebot */
+    } else if (current->next != NULL && current->next != filebot) {
 	current->data = nrealloc(current->data,
 				 strlen(current->data) +
 				 strlen(current->next->data) + 1);
@@ -1078,7 +1154,7 @@ int do_delete(void)
 	delete_node(foo);
 	update_line(current, current_x);
 
-	/* Please see the comment in do_basckspace if you don't understand
+	/* Please see the comment in do_backspace if you don't understand
 	   this test */
 	if (current == filebot && strcmp(current->data, "")) {
 	    new_magicline();
@@ -1102,7 +1178,7 @@ void wrap_reset(void)
     UNSET(SAMELINEWRAP);
 }
 
-#if !defined(NANO_SMALL) && !defined(DISABLE_SPELLER)
+#ifndef DISABLE_SPELLER
 
 int do_int_spell_fix(char *word)
 {
@@ -1168,9 +1244,6 @@ int do_int_spell_fix(char *word)
 
     return TRUE;
 }
-#endif
-
-#if !defined(NANO_SMALL) && !defined(DISABLE_SPELLER)
 
 /* Integrated spell checking using 'spell' program */
 int do_int_speller(char *tempfile_name)
@@ -1298,24 +1371,37 @@ int do_int_speller(char *tempfile_name)
 
     return TRUE;
 }
-#endif
-
-#if !defined(NANO_SMALL) && !defined(DISABLE_SPELLER)
 
 /* External spell checking */
 int do_alt_speller(char *file_name)
 {
     int alt_spell_status;
     pid_t pid_spell;
+    char *ptr;
+    static int arglen = 3;
+    static char **spellargs = (char **) NULL;
 
     endwin();
 
-    /* Start a new process for the alternate speller */
+    /* Set up an argument list to pass the execvp function */
+    if (spellargs == NULL) {
+	spellargs = nmalloc(arglen * sizeof(char *));
 
+	spellargs[0] = strtok(alt_speller, " ");
+	while ((ptr = strtok(NULL, " ")) != NULL) {
+	    arglen++;
+	    spellargs = nrealloc(spellargs, arglen * sizeof(char *));
+	    spellargs[arglen - 3] = ptr;
+	}
+	spellargs[arglen - 1] = NULL;
+    }
+    spellargs[arglen - 2] = file_name;
+
+    /* Start a new process for the alternate speller */
     if ( (pid_spell = fork()) == 0) {
 
 	/* Start alternate spell program, we are using the PATH here!?!? */
-	execlp(alt_speller, alt_speller, file_name, NULL);
+	execvp(spellargs[0], spellargs);
 
 	/* Should not be reached, if alternate speller is found!!! */
 
@@ -1352,10 +1438,7 @@ int do_alt_speller(char *file_name)
 int do_spell(void)
 {
 
-#if defined(NANO_SMALL)
-    nano_small_msg();
-    return (TRUE);
-#elif defined(DISABLE_SPELLER)
+#ifdef DISABLE_SPELLER
     nano_disabled_msg();
     return (TRUE);
 #else
@@ -1411,7 +1494,7 @@ int do_exit(void)
 #endif
 
     if (i == 1) {
-	if (do_writeout(1) > 0)
+	if (do_writeout(filename, 1) > 0)
 	    finish(0);
     } else if (i == 0)
 	finish(0);
@@ -1532,7 +1615,6 @@ void handle_sigwinch(int s)
     char *tty = NULL;
     int fd = 0;
     int result = 0;
-    int i = 0;
     struct winsize win;
 
     tty = ttyname(0);
@@ -1549,19 +1631,17 @@ void handle_sigwinch(int s)
     COLS = win.ws_col;
     LINES = win.ws_row;
 
-    center_x = COLS / 2;
-    center_y = LINES / 2;
-    editwinrows = LINES - 5 + no_help();
-    fill = COLS - 8;
+    if ((editwinrows = LINES - 5 + no_help()) < MIN_EDITOR_ROWS)
+	die_too_small();
 
-    free(hblank);
-    hblank = nmalloc(COLS + 1);
+    if ((fill = COLS - CHARS_FROM_EOL) < MIN_FILL_LENGTH)
+	die_too_small();
 
-    for (i = 0; i <= COLS - 1; i++)
-	hblank[i] = ' ';
-    hblank[i] = 0;
+    hblank = nrealloc(hblank, COLS + 1);
+    memset(hblank, ' ', COLS);
+    hblank[COLS] = 0;
 
-#ifdef HAVE_NCURSES_H
+#ifdef HAVE_RESIZETERM
     resizeterm(LINES, COLS);
 #ifdef HAVE_WRESIZE
     if (wresize(topwin, 2, COLS) == ERR)
@@ -1577,7 +1657,7 @@ void handle_sigwinch(int s)
     if (mvwin(bottomwin, LINES - 3 + no_help(), 0) == ERR)
 	die(_("Cannot move bottom win"));
 #endif				/* HAVE_WRESIZE */
-#endif				/* HAVE_NCURSES_H */
+#endif				/* HAVE_RESIZETERM */
 
     fix_editbot();
 
@@ -1588,10 +1668,18 @@ void handle_sigwinch(int s)
 
     /* Do these b/c width may have changed... */
     refresh();
-    titlebar();
+    titlebar(NULL);
     edit_refresh();
     display_main_list();
+    blank_statusbar();
     total_refresh();
+
+    /* Turn cursor back on for sure */
+    curs_set(1);
+
+    /* Jump back to mainloop */
+    siglongjmp(jmpbuf, 1);
+
 #endif
 }
 
@@ -1625,7 +1713,8 @@ void signal_init(void)
 
 void window_init(void)
 {
-    editwinrows = LINES - 5 + no_help();
+    if ((editwinrows = LINES - 5 + no_help()) < MIN_EDITOR_ROWS)
+	die_too_small();
 
     /* Setup up the main text window */
     edit = newwin(editwinrows, COLS, 2, 0);
@@ -1633,6 +1722,14 @@ void window_init(void)
     /* And the other windows */
     topwin = newwin(2, COLS, 0, 0);
     bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);
+
+#ifdef PDCURSES
+    /* Oops, I guess we need this again.
+       Moved here so the keypad still works after a Meta-X, for example */
+    keypad(edit, TRUE);
+    keypad(bottomwin, TRUE);
+#endif
+
 }
 
 void mouse_init(void)
@@ -1640,15 +1737,14 @@ void mouse_init(void)
 #ifndef NANO_SMALL
 #ifdef NCURSES_MOUSE_VERSION
     if (ISSET(USE_MOUSE)) {
-	keypad_on(TRUE);
+	keypad_on(edit, 1);
 
 	mousemask(BUTTON1_RELEASED, NULL);
 	mouseinterval(50);
 
-    } else {
+    } else
 	mousemask(0, NULL);
-	keypad_on(FALSE);
-    }
+
 #endif
 #endif
 
@@ -1660,7 +1756,7 @@ int do_tab(void)
     return 1;
 }
 
-#if !defined(NANO_SMALL) && !defined(DISABLE_JUSTIFY)
+#ifndef DISABLE_JUSTIFY
 int empty_line(const char *data)
 {
     while (*data) {
@@ -1712,15 +1808,12 @@ void justify_format(char *data)
 
 int do_justify(void)
 {
-#ifdef NANO_SMALL
-    nano_small_msg();
-    return 1;
-#elif defined(DISABLE_JUSTIFY)
+#ifdef DISABLE_JUSTIFY
     nano_disabled_msg();
     return 1;
 #else
     int slen = 0;		/* length of combined lines on one line. */
-    int initial_y, kbinput;
+    int initial_y, kbinput = 0, totbak;
     filestruct *initial = NULL, *tmpjust = NULL, *cutbak, *tmptop, *tmpbot;
 
     if (empty_line(current->data)) {
@@ -1760,11 +1853,15 @@ int do_justify(void)
 
     set_modified();
     cutbak = cutbuffer; /* Got to like cutbak ;) */
+    totbak = totsize;
     cutbuffer = NULL;
 
     tmptop = current;
     tmpjust = copy_node(current);
+
+    /* This is annoying because it mucks with totsize */
     add_to_cutbuffer(tmpjust);
+
     /* Put the whole paragraph into one big line. */
     while (current->next && !isspace((int) current->next->data[0])
 	   && current->next->data[0]) {
@@ -1776,6 +1873,9 @@ int do_justify(void)
 	tmpjust = copy_node(current->next);
 	add_to_cutbuffer(tmpjust);
 
+	/* Wiping out a newline */
+        totsize--;
+
 	/* length of both strings plus space between strings and ending \0. */
 	current->data = nrealloc(current->data, len + len2 + 2);
 	current->data[len++] = ' ';
@@ -1787,8 +1887,6 @@ int do_justify(void)
 	delete_node(tmpnode);
     }
 
-    totsize -= strlen(current->data);
-
     justify_format(current->data);
 
     slen = strlen(current->data);
@@ -1808,12 +1906,14 @@ int do_justify(void)
 		i = fill;
 	    else
 		i = slen;
+
 	    for (; i > 0; i--) {
 		if (isspace((int) current->data[i]) &&
 		    ((strlenpt(current->data) - strlen(current->data + i))
 		     <= fill))
 		    break;
 	    }
+
 	    if (!i)
 		break;
 
@@ -1871,9 +1971,10 @@ int do_justify(void)
 
     /* Now get a keystroke and see if it's unjustify, if not unget the keytroke 
        and return */
-    if ((kbinput = wgetch(edit)) != NANO_UNJUSTIFY_KEY)
+    if ((kbinput = wgetch(edit)) != NANO_UNJUSTIFY_KEY) {
 	ungetch(kbinput); 
-    else {
+	blank_statusbar_refresh();
+    } else {
 	/* Else restore the justify we just did (ungrateful user!) */
 	if (tmptop->prev != NULL)
 	    tmptop->prev->next = tmpbot->next;
@@ -1886,6 +1987,8 @@ int do_justify(void)
 	if (tmptop->prev == NULL)
 	    edit_refresh();
 
+	/* Restore totsize from befure justify */
+	totsize = totbak;
 	free_filestruct(tmptop);
 	blank_statusbar_refresh();
     }
@@ -1898,16 +2001,16 @@ int do_justify(void)
 #endif
 }
 
-
+#ifndef DISABLE_HELP
 void help_init(void)
 {
     int i, sofar = 0;
     long allocsize = 1;		/* How much space we're gonna need for the help text */
-    char buf[BUFSIZ];
+    char buf[BUFSIZ] = "";
 
     /* Compute the space needed for the shortcut lists - we add 15 to
        have room for the shortcut abbrev and its possible alternate keys */
-    for (i = 0; i < MAIN_LIST_LEN; i++)
+    for (i = 0; i <= MAIN_LIST_LEN - 1; i++)
 	if (main_list[i].help != NULL)
 	    allocsize += strlen(main_list[i].help) + 15;
 
@@ -1928,7 +2031,7 @@ void help_init(void)
     strcpy(help_text, help_text_init);
 
     /* Now add our shortcut info */
-    for (i = 0; i < MAIN_LIST_LEN - 1; i++) {
+    for (i = 0; i <= MAIN_LIST_LEN - 1; i++) {
 	sofar = snprintf(buf, BUFSIZ, "^%c	", main_list[i].val + 64);
 
 	if (main_list[i].misc1 > KEY_F0 && main_list[i].misc1 <= KEY_F(64))
@@ -1966,10 +2069,13 @@ void help_init(void)
     }
 
 }
+#endif
 
 void do_toggle(int which)
 {
-#ifndef NANO_SMALL
+#ifdef NANO_SMALL
+    nano_disabled_msg();
+#else
     char *enabled = _("enabled");
     char *disabled = _("disabled");
 
@@ -1993,6 +2099,7 @@ void do_toggle(int which)
 	wclear(bottomwin);
 	wrefresh(bottomwin);
 	window_init();
+	fix_editbot();
 	edit_refresh();
 	display_main_list();
 	break;
@@ -2013,18 +2120,47 @@ void do_toggle(int which)
     }
     SET(DISABLE_CURPOS);
 
-#else
-    nano_small_msg();
 #endif
 }
 
+/* If the NumLock key has made the keypad gone awry, print an error
+   message, hopefully we can address it later. */
+void print_numlock_warning(void)
+{
+    static int didmsg = 0;
+    if (!didmsg) {
+	statusbar(_("NumLock glitch detected.  Keypad will malfunction with NumLock off"));
+	didmsg = 1;
+    }
+}
+
+/* This function returns the correct keystroke, given the A,B,C or D
+   input key.  This is a common sequence of many terms which send
+   Esc-O-[A-D] or Esc-[-[A-D]. */
+int ABCD(int input)
+{
+    switch(input)
+    {                
+	case 'A':
+	    return(KEY_UP);
+	case 'B':
+	    return(KEY_DOWN);
+	case 'C':
+	    return(KEY_RIGHT);
+	case 'D': 
+	    return(KEY_LEFT);
+	default:
+	    return 0;
+    }
+}
+
 int main(int argc, char *argv[])
 {
     int optchr;
     int kbinput;		/* Input from keyboard */
     long startline = 0;		/* Line to try and start at */
-    int keyhandled = 0;		/* Have we handled the keystroke yet? */
-    int i;
+    int keyhandled;		/* Have we handled the keystroke yet? */
+    int i, modify_control_seq;
     char *argv0;
 #ifdef _POSIX_VDISABLE
     struct termios term;
@@ -2042,12 +2178,15 @@ int main(int argc, char *argv[])
 	{"nowrap", 0, 0, 'w'},
 	{"nohelp", 0, 0, 'x'},
 	{"help", 0, 0, 'h'},
+	{"view", 0, 0, 'v'},
 #ifndef NANO_SMALL
 	{"cut", 0, 0, 'k'},
 #endif
 	{"autoindent", 0, 0, 'i'},
 	{"tempfile", 0, 0, 't'},
+#ifndef DISABLE_SPELLER
 	{"speller", 1, 0, 's'},
+#endif
 	{"fill", 1, 0, 'r'},
 	{"mouse", 0, 0, 'm'},
 	{"pico", 0, 0, 'p'},
@@ -2061,16 +2200,18 @@ int main(int argc, char *argv[])
     SET(FOLLOW_SYMLINKS);
 
 #ifndef NANO_SMALL
+#ifdef ENABLE_NLS
     setlocale(LC_ALL, "");
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
 #endif
+#endif
 
 #ifdef HAVE_GETOPT_LONG
-    while ((optchr = getopt_long(argc, argv, "?T:RVchiklmpr:s:tvwxz",
+    while ((optchr = getopt_long(argc, argv, "?T:RVbcefhiklmpr:s:tvwxz",
 				 long_options, &option_index)) != EOF) {
 #else
-    while ((optchr = getopt(argc, argv, "h?T:RVciklmpr:s:tvwxz")) != EOF) {
+    while ((optchr = getopt(argc, argv, "h?T:RVbcefiklmpr:s:tvwxz")) != EOF) {
 #endif
 
 	switch (optchr) {
@@ -2089,6 +2230,11 @@ int main(int argc, char *argv[])
 	case 'V':
 	    version();
 	    exit(0);
+	case 'b':
+	case 'e':
+	case 'f':
+		/* Pico compatibility flags */
+		break;
 	case 'c':
 	    SET(CONSTUPDATE);
 	    break;
@@ -2103,10 +2249,6 @@ int main(int argc, char *argv[])
 	case 'k':
 	    SET(CUT_TO_END);
 	    break;
-#else
-	case 'k':
-	    usage();		/* Oops!  You dont really have that option */
-	    finish(1);
 #endif
 	case 'l':
 	    UNSET(FOLLOW_SYMLINKS);
@@ -2124,10 +2266,12 @@ int main(int argc, char *argv[])
 		finish(1);
 	    }
 	    break;
+#ifndef DISABLE_SPELLER
 	case 's':
 	    alt_speller = nmalloc(strlen(optarg) + 1);
 	    strcpy(alt_speller, optarg);
 	    break;
+#endif
 	case 't':
 	    SET(TEMP_OPT);
 	    break;
@@ -2158,19 +2302,19 @@ int main(int argc, char *argv[])
     /* See if there's a non-option in argv (first non-option is the
        filename, if +LINE is not given) */
     if (argc == 1 || argc <= optind)
-	strcpy(filename, "");
+	clear_filename();
     else {
 	/* Look for the +line flag... */
 	if (argv[optind][0] == '+') {
 	    startline = atoi(&argv[optind][1]);
 	    optind++;
 	    if (argc == 1 || argc <= optind)
-		strcpy(filename, "");
+		clear_filename();
 	    else
-		strncpy(filename, argv[optind], 132);
-	} else
-	    strncpy(filename, argv[optind], 132);
+		filename = mallocstrcpy(filename, argv[optind]);
 
+	} else
+	    filename = mallocstrcpy(filename, argv[optind]);
     }
 
 
@@ -2191,13 +2335,14 @@ int main(int argc, char *argv[])
     nonl();
     cbreak();
     noecho();
-    timeout(0);
 
     /* Set up some global variables */
     global_init();
     shortcut_init(0);
+#ifndef DISABLE_HELP
     init_help_msg();
     help_init();
+#endif
     signal_init();
 
 #ifdef DEBUG
@@ -2207,12 +2352,6 @@ int main(int argc, char *argv[])
     window_init();
     mouse_init();
 
-#ifdef PDCURSES
-    /* Must have this for the arrow, et al, keys to even work in 
-       PDCurses+cygwin under Windows */
-    keypad_on(TRUE);
-#endif
-
 #ifdef DEBUG
     fprintf(stderr, _("Main: bottom win\n"));
 #endif
@@ -2223,7 +2362,7 @@ int main(int argc, char *argv[])
     fprintf(stderr, _("Main: open file\n"));
 #endif
 
-    titlebar();
+    titlebar(NULL);
 
     /* Now we check to see if argv[optind] is non-null to determine if
        we're dealing with a new file or not, not argc == 1... */
@@ -2237,6 +2376,14 @@ int main(int argc, char *argv[])
     else
 	edit_update(fileage, CENTER);
 
+    /* return here after a sigwinch */
+    sigsetjmp(jmpbuf,1);
+
+    /* Fix clobber-age */
+    kbinput = 0;
+    keyhandled = 0;
+    modify_control_seq = 0;
+
     edit_refresh();
     reset_cursor();
 
@@ -2248,12 +2395,19 @@ int main(int argc, char *argv[])
 #endif
 
 	kbinput = wgetch(edit);
+#ifdef DEBUG
+	fprintf(stderr, "AHA!  %c (%d)\n", kbinput, kbinput);
+#endif
 	if (kbinput == 27) {	/* Grab Alt-key stuff first */
 	    switch (kbinput = wgetch(edit)) {
 		/* Alt-O, suddenly very important ;) */
 	    case 79:
 		kbinput = wgetch(edit);
-		if (kbinput <= 'S' && kbinput >= 'P')
+		if (kbinput <= 'D' && kbinput >= 'A')
+		   kbinput = ABCD(kbinput);
+		else if (kbinput <= 'z' && kbinput >= 'j')
+		    print_numlock_warning();
+		else if (kbinput <= 'S' && kbinput >= 'P')
 		    kbinput = KEY_F(kbinput - 79);
 #ifdef DEBUG
 		else {
@@ -2263,8 +2417,13 @@ int main(int argc, char *argv[])
 		}
 #endif
 		break;
+	    case 27:
+		/* If we get Alt-Alt, the next keystroke should be the same as a
+		   control sequence */
+		modify_control_seq = 1;
+		keyhandled = 1;
+		break;
 	    case 91:
-
 		switch (kbinput = wgetch(edit)) {
 		case '1':	/* Alt-[-1-[0-5,7-9] = F1-F8 in X at least */
 		    kbinput = wgetch(edit);
@@ -2341,16 +2500,10 @@ int main(int argc, char *argv[])
 			kbinput = KEY_F(kbinput - 64);
 		    break;
 		case 'A':
-		    kbinput = KEY_UP;
-		    break;
 		case 'B':
-		    kbinput = KEY_DOWN;
-		    break;
 		case 'C':
-		    kbinput = KEY_RIGHT;
-		    break;
 		case 'D':
-		    kbinput = KEY_LEFT;
+		    kbinput = ABCD(kbinput);
 		    break;
 		case 'H':
 		    kbinput = KEY_HOME;
@@ -2392,6 +2545,18 @@ int main(int argc, char *argv[])
 		break;
 	    }
 	}
+	/* If the modify_control_seq is set, we received an Alt-Alt 
+	   sequence before this, so we make this key a control sequence 
+	   by subtracting 64 or 96, depending on its value. */
+	if (!keyhandled && modify_control_seq) {
+	    if (kbinput >= 'A' && kbinput < 'a')
+		kbinput -= 64;
+	    else if (kbinput >= 'a' && kbinput <= 'z')
+		kbinput -= 96;
+
+	    modify_control_seq = 0;
+	}
+
 	/* Look through the main shortcut list to see if we've hit a
 	   shortcut key */
 	for (i = 0; i < MAIN_LIST_LEN && !keyhandled; i++) {
@@ -2405,18 +2570,17 @@ int main(int argc, char *argv[])
 		keyhandled = 1;
 	    }
 	}
-#ifndef _POSIX_VDISABLE
-	/* Since we're in raw mode, we have to catch ^Q and ^S */
+	/* If we're in raw mode or using Alt-Alt-x, we have to catch
+	   Control-S and Control-Q */
 	if (kbinput == 17 || kbinput == 19)
 	    keyhandled = 1;
 
-	/* And catch ^Z by hand when triggered */
+	/* Catch ^Z by hand when triggered also */
 	if (kbinput == 26) {
 	    if (ISSET(SUSPEND))
 		do_suspend(0);
 	    keyhandled = 1;
 	}
-#endif
 
 	/* Last gasp, stuff that's not in the main lists */
 	if (!keyhandled)
@@ -2431,6 +2595,7 @@ int main(int argc, char *argv[])
 	    case 0:		/* Erg */
 		do_next_word();
 		break;
+
 	    case 331:		/* Stuff that we don't want to do squat */
 	    case -1:
 	    case 410:		/* Must ignore this, it gets sent when we resize */
