@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.14 2000/11/30 02:31:13 astyanax Exp $ */
+/* $Id: utils.c,v 1.21 2001/03/18 18:55:06 astyanax Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -123,9 +123,13 @@ void *nrealloc(void *ptr, size_t howmuch)
 
    Should be used as dest = mallocstrcpy(dest, src);
 */
-void *mallocstrcpy(void *dest, void *src)
+void *mallocstrcpy(char *dest, char *src)
 {
 
+
+    if (src == dest)
+	return src;
+
     if (dest != NULL)
 	free(dest);
 
@@ -152,6 +156,7 @@ void new_magicline(void)
     filebot->next->lineno = filebot->lineno + 1;
     filebot = filebot->next;
     totlines++;
+    totsize++;
 }
 
 #ifndef DISABLE_TABCOMP
