@@ -1,4 +1,4 @@
-/* $Id: winio.c,v 1.62 2000/12/05 11:36:41 astyanax Exp $ */
+/* $Id: winio.c,v 1.83 2001/03/22 23:12:22 astyanax Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -34,6 +34,8 @@
 #define _(string) (string)
 #endif
 
+
+/* winio.c statics */
 static int statblank = 0;	/* Number of keystrokes left after
 				   we call statubar() before we
 				   actually blank the statusbar */
@@ -308,6 +310,7 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 	case KEY_RIGHT:
+	case NANO_FORWARD_KEY:
 
 	    if (x < xend)
 		x++;
@@ -353,11 +356,14 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 		inputbuf = input_tab(inputbuf, (x - x_left), 
 				&tabbed, &shift);
 		x += shift;
+		if (x - x_left > strlen(inputbuf))
+		    x = strlen(inputbuf) + x_left;
 		nanoget_repaint(buf, inputbuf, x);
 	    }
 	    break;
 #endif
 	case KEY_LEFT:
+	case NANO_BACK_KEY:
 	    if (x > strlen(buf))
 		x--;
 	    wmove(bottomwin, 0, x);
@@ -463,9 +469,13 @@ void horizbar(WINDOW * win, int y)
     wattroff(win, A_REVERSE);
 }
 
-void titlebar(void)
+void titlebar(char *path)
 {
     int namelen, space;
+    char *what = path;
+
+    if (path == NULL)
+	what = filename;
 
     horizbar(topwin, 0);
     wattron(topwin, A_REVERSE);
@@ -473,17 +483,23 @@ void titlebar(void)
 
     space = COLS - strlen(VERMSG) - strlen(VERSION) - 21;
 
-    namelen = strlen(filename);
+    namelen = strlen(what);
 
-    if (!strcmp(filename, ""))
-	mvwaddstr(topwin, 0, center_x - 6, _("New Buffer"));
+    if (!strcmp(what, ""))
+	mvwaddstr(topwin, 0, COLS / 2 - 6, _("New Buffer"));
     else {
 	if (namelen > space) {
-	    waddstr(topwin, _("  File: ..."));
-	    waddstr(topwin, &filename[namelen - space]);
+	    if (path == NULL)
+		waddstr(topwin, _("  File: ..."));
+	    else
+		waddstr(topwin, _("   DIR: ..."));
+	    waddstr(topwin, &what[namelen - space]);
 	} else {
-	    mvwaddstr(topwin, 0, center_x - (namelen / 2 + 1), "File: ");
-	    waddstr(topwin, filename);
+	    if (path == NULL)
+		mvwaddstr(topwin, 0, COLS / 2 - (namelen / 2 + 1), "File: ");
+	    else
+	 	mvwaddstr(topwin, 0, COLS / 2 - (namelen / 2 + 1), " DIR: ");
+	    waddstr(topwin, what);
 	}
     }
     if (ISSET(MODIFIED))
@@ -552,7 +568,7 @@ void set_modified(void)
 {
     if (!ISSET(MODIFIED)) {
 	SET(MODIFIED);
-	titlebar();
+	titlebar(NULL);
 	wrefresh(topwin);
     }
 }
@@ -1009,7 +1025,18 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
 {
     va_list ap;
     char foo[133];
-    int kbinput, ok = -1;
+    int kbinput, ok = -1, i;
+    char *yesstr;		/* String of yes characters accepted */
+    char *nostr;		/* Same for no */
+    char *allstr;		/* And all, surprise! */
+    char shortstr[5];		/* Temp string for above */
+
+    /* Yes, no and all are strings of any length.  Each string consists of
+	all characters accepted as a valid character for that value.
+	The first value will be the one displayed in the shortcuts. */
+    yesstr = _("Yy");
+    nostr = _("Nn");
+    allstr = _("Aa");
 
     /* Write the bottom of the screen */
     clear_bottomwin();
@@ -1020,11 +1047,19 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
     /* Remove gettext call for keybindings until we clear the thing up */
     if (!ISSET(NO_HELP)) {
 	wmove(bottomwin, 1, 0);
-	onekey(" Y", _("Yes"));
-	if (all)
-	    onekey(" A", _("All"));
+
+	snprintf(shortstr, 3, " %c", yesstr[0]);
+	onekey(shortstr, _("Yes"));
+
+	if (all) {
+	    snprintf(shortstr, 3, " %c", allstr[0]);
+	    onekey(shortstr, _("All"));
+	}
 	wmove(bottomwin, 2, 0);
-	onekey(" N", _("No"));
+
+	snprintf(shortstr, 3, " %c", nostr[0]);
+	onekey(shortstr, _("No"));
+
 	onekey("^C", _("Cancel"));
     }
     va_start(ap, msg);
@@ -1042,22 +1077,34 @@ int do_yesno(int all, int leavecursor, char *msg, ...)
 	kbinput = wgetch(edit);
 
 	switch (kbinput) {
-	case 'Y':
-	case 'y':
-	    ok = 1;
-	    break;
-	case 'N':
-	case 'n':
-	    ok = 0;
-	    break;
-	case 'A':
-	case 'a':
-	    if (all)
-		ok = 2;
-	    break;
 	case NANO_CONTROL_C:
 	    ok = -2;
 	    break;
+	default:
+
+	    /* Look for the kbinput in the yes, no and (optinally) all str */
+	    for (i = 0; yesstr[i] != 0 && yesstr[i] != kbinput; i++)
+		;
+	    if (yesstr[i] != 0) {
+		ok = 1;
+	 	break;
+	    }
+
+	    for (i = 0; nostr[i] != 0 && nostr[i] != kbinput; i++)
+		;
+	    if (nostr[i] != 0) {
+		ok = 0;
+	 	break;
+	    }
+
+	    if (all) {
+	        for (i = 0; allstr[i] != 0 && allstr[i] != kbinput; i++)
+		    ;
+		if (allstr[i] != 0) {
+		    ok = 2;
+	 	    break;
+		}
+	    }
 	}
     }
 
@@ -1080,7 +1127,7 @@ void statusbar(char *msg, ...)
     vsnprintf(foo, 132, msg, ap);
     va_end(ap);
 
-    start_x = center_x - strlen(foo) / 2 - 1;
+    start_x = COLS / 2 - strlen(foo) / 2 - 1;
 
     /* Blank out line */
     blank_statusbar();
@@ -1119,7 +1166,7 @@ int total_refresh(void)
     clearok(topwin, FALSE);
     clearok(bottomwin, FALSE);
     edit_refresh();
-    titlebar();
+    titlebar(NULL);
     return 1;
 }
 
@@ -1132,33 +1179,26 @@ void previous_line(void)
 int do_cursorpos(void)
 {
     filestruct *fileptr;
-    float linepct, bytepct;
-    int i, tot = 0;
+    float linepct = 0.0, bytepct = 0.0;
+    int i = 0;
 
     if (current == NULL || fileage == NULL)
 	return 0;
 
     for (fileptr = fileage; fileptr != current && fileptr != NULL;
 	 fileptr = fileptr->next)
-	tot += strlen(fileptr->data) + 1;
+	i += strlen(fileptr->data) + 1;
 
     if (fileptr == NULL)
 	return -1;
 
-    i = tot + current_x;;
-
-    for (fileptr = current->next; fileptr != NULL; fileptr = fileptr->next)
-	tot += strlen(fileptr->data) + 1;
+    i += current_x;
 
     if (totlines > 0)
 	linepct = 100 * current->lineno / totlines;
-    else
-	linepct = 0;
 
     if (totsize > 0)
 	bytepct = 100 * i / totsize;
-    else
-	bytepct = 0;
 
 #ifdef DEBUG
     fprintf(stderr, _("do_cursorpos: linepct = %f, bytepct = %f\n"),
@@ -1175,25 +1215,26 @@ int do_cursorpos(void)
    But hey, it's better than nothing, and it's dynamic! */
 int do_help(void)
 {
-#ifndef NANO_SMALL
+#ifndef DISABLE_HELP
     char *ptr = help_text, *end;
-    int i, j, row = 0, page = 1, kbinput = 0, no_more = 0;
+    int i, j, row = 0, page = 1, kbinput = 0, no_more = 0, kp;
     int no_help_flag = 0;
 
     blank_edit();
     curs_set(0);
     blank_statusbar();
 
+    kp = keypad_on(edit, 1);
+
     if (ISSET(NO_HELP)) {
 
+	/* Well if we're going to do this, we should at least
+	   do it the right way */
 	no_help_flag = 1;
-	delwin(bottomwin);
-	bottomwin = newwin(3, COLS, LINES - 3, 0);
-	keypad(bottomwin, TRUE);
-
-	editwinrows -= no_help();
 	UNSET(NO_HELP);
+	window_init();
 	bottombars(help_list, HELP_LIST_LEN);
+
     } else
 	bottombars(help_list, HELP_LIST_LEN);
 
@@ -1265,22 +1306,23 @@ int do_help(void)
 	    no_more = 1;
 	    continue;
 	}
-    } while ((kbinput = wgetch(edit)) != NANO_EXIT_KEY);
+    } while ((kbinput = wgetch(edit)) != NANO_EXIT_KEY && 
+      kbinput != NANO_EXIT_FKEY);
+
     if (no_help_flag) {
-	werase(bottomwin);
+	blank_bottombars();
 	wrefresh(bottomwin);
-	delwin(bottomwin);
 	SET(NO_HELP);
-	bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);
-	keypad(bottomwin, TRUE);
-	editwinrows += no_help();
-    } else
-	display_main_list();
+	window_init();
+    }
+    display_main_list();
 
     curs_set(1);
     edit_refresh();
-#else
-    nano_small_msg();
+    kp = keypad_on(edit, kp);
+
+#elif defined(DISABLE_HELP)
+    nano_disabled_msg();
 #endif
 
     return 1;
@@ -1366,7 +1408,7 @@ void do_replace_highlight(int highlight_flag, char *word)
 }
 
 #ifdef NANO_EXTRA
-#define CREDIT_LEN 45
+#define CREDIT_LEN 47
 void do_credits(void)
 {
     int i, j = 0, k, place = 0, start_x;
@@ -1400,9 +1442,11 @@ void do_credits(void)
 			"Tedi Heriyanto",
 			"Bill Soudan",
 			"Christian Weisgerber",
-			"Erik Anderson",
+			"Erik Andersen",
 			"Big Gaute",
 			"Joshua Jensen",
+			"Ryan Krebs",
+			"Albert Chin",
 			"",
 			specialthx,
 			"Plattsburgh State University",
@@ -1442,7 +1486,7 @@ void do_credits(void)
 		else
 		    what = "";
 
-		start_x = center_x - strlen(what) / 2 - 1;
+		start_x = COLS / 2 - strlen(what) / 2 - 1;
 		mvwaddstr(edit, i * 2 - k, start_x, what);
 	    }
 	    usleep(700000);
@@ -1463,3 +1507,23 @@ void do_credits(void)
     total_refresh();
  }
 #endif
+
+int             keypad_on(WINDOW * win, int newval)
+{
+
+/* This is taken right from aumix.  Don't sue me */
+#ifdef HAVE_USEKEYPAD
+    int             old;
+
+    old = win->_use_keypad;
+    keypad(win, newval);
+    return old;
+#else
+    keypad(win, newval);
+    return 1;
+#endif                          /* HAVE_USEKEYPAD */
+
+}
+
+
+
